@@ -1,5 +1,6 @@
 require 'puppet_x/puppetlabs/tune/calculate'
 require 'puppet_x/puppetlabs/tune/inventory'
+require 'puppet_x/puppetlabs/tune/optional_services'
 require 'puppet_x/puppetlabs/tune/peconf'
 require 'puppet_x/puppetlabs/tune/query'
 
@@ -31,6 +32,10 @@ module PuppetX
           'puppet_enterprise::profile::orchestrator::jruby_max_active_instances',
           'puppet_enterprise::profile::puppetdb::java_args',
           'puppet_enterprise::puppetdb::command_processing_threads',
+          'puppet_enterprise::master::host_action_collector::java_args',
+          'puppet_enterprise::profile::patching_service::java_args',
+          'puppet_enterprise::profile::infra_assistant::java_args',
+          'puppet_enterprise::profile::workflow_service::java_args',
         ]
         param_names
       end
@@ -49,6 +54,9 @@ module PuppetX
           'primary_master_replica',
           'enabled_primary_master_replica',
           'compile_master',
+          'patching_service',
+          'infra_assistant',
+          'workflow_service',
         ]
         class_names
       end
@@ -124,6 +132,7 @@ module PuppetX
         @pe_conf = PuppetX::Puppetlabs::Tune::PEConf.new('/etc/puppetlabs/enterprise') if @options[:pe_conf]
         @inventory = PuppetX::Puppetlabs::Tune::Inventory.new
         @query = PuppetX::Puppetlabs::Tune::Query.new
+        @optional_services = PuppetX::Puppetlabs::Tune::OptionalServices.new(@query, @inventory)
       end
 
       # Output the current infrastructure summary.
@@ -383,6 +392,9 @@ module PuppetX
         node['infrastructure'] = {
           'is_monolithic'        => monolithic?,
           'with_compile_masters' => with_compile_masters?,
+          'has_patching_service' => patching_service_enabled?,
+          'has_infra_assistant'  => infra_assistant_enabled?,
+          'has_workflow_service' => workflow_service_enabled?,
         }
         node['resources'] = resources
         node['type'] = {
@@ -904,6 +916,23 @@ module PuppetX
 
       # Identify a PE Infrastructure node.
 
+      # @return [String] The certname of the primary master.
+      #
+      # @note This should be singular if determined from config/pdb (there is
+      # no current architecture supporting multiple primaries). However an
+      # inventory file could be written with a plurality of primaries. Testing
+      # the state of the optional patching service, as an example, is a case
+      # where we have to know the singular primary.
+      def primary
+        primaries = @nodes_with_role['primary_masters']
+        the_primary = primaries.first
+        if primaries.count > 1
+          msg = _("Ambiguous primaries detected: %{primaries}. Assuming %{the_primary} to be the primary." % { primaries: primaries, the_primary: the_primary })
+          Puppet.warn(msg)
+        end
+        the_primary
+      end
+
       def monolithic_master?(certname)
         monolithic? && @nodes_with_role['primary_masters'].include?(certname)
       end
@@ -937,6 +966,20 @@ module PuppetX
         classes
       end
 
+      # Identify status of optional primary services.
+
+      def patching_service_enabled?
+        @optional_services.enabled?(primary, :patching)
+      end
+
+      def infra_assistant_enabled?
+        @optional_services.enabled?(primary, :infra_assistant)
+      end
+
+      def workflow_service_enabled?
+        @optional_services.enabled?(primary, :workflow_service)
+      end
+
       # Identify available jrubies on a PE Infrastructure node.
       # Used when estimating PE Infrastructure capacity.
 
